package com.cn.esermis.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.cn.esermis.dpld.entity.AnalyticalInstruments;
import com.cn.esermis.dpld.entity.DataAcquisitionInstrument;
import com.cn.esermis.dpld.entity.DetectMonitoringFactor;
import com.cn.esermis.dpld.model.AjaxResult;
import com.cn.esermis.dpld.model.AnalyticalInstrumentsVo;
import com.cn.esermis.dpld.model.DataAcquisitionInstrumentVo;
import com.cn.esermis.dpld.service.IAnalyticalInstrumentsService;
import com.cn.esermis.dpld.service.IDataAcquisitionInstrumentService;
import com.cn.esermis.dpld.service.IDetectMonitoringFactorService;
import org.springframework.beans.BeanUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import static com.cn.esermis.dpld.model.AjaxResult.success;

@RestController
@RequestMapping("/data_Acquisition")
public class DataAcquisitionInstrumentController {
    @Resource
    private IAnalyticalInstrumentsService analyticalInstrumentsService;
    @Resource
    private IDetectMonitoringFactorService detectMonitoringFactorService;
    @Resource
    private IDataAcquisitionInstrumentService dataAcquisitionInstrumentService;

    /**
     * 获取菜单列表
     */
    @GetMapping("/list")
    public AjaxResult list(DataAcquisitionInstrumentVo vo)
    {
        IPage<DataAcquisitionInstrument> page = new Page<>(vo.getPageNum(), vo.getPageSize());
        QueryWrapper<DataAcquisitionInstrument> wrapper = new QueryWrapper<>();
        IPage<DataAcquisitionInstrument> pageList = dataAcquisitionInstrumentService.page(page, wrapper);
        IPage<DataAcquisitionInstrumentVo> volits = new Page<>();
        if (!pageList.getRecords().isEmpty()){
            volits.setTotal(pageList.getTotal());
            volits.setCurrent(pageList.getCurrent());
            volits.setSize(pageList.getSize());
            List<DataAcquisitionInstrument> records = pageList.getRecords();
            List<DataAcquisitionInstrumentVo> revoList = new ArrayList<>();
            // 查询企业
            Map<String,Object> componyMap =  dataAcquisitionInstrumentService.getComponyMap();
            // 查询工厂
            Map<String, Object> departmentMap = dataAcquisitionInstrumentService.getDepartmentMap();
            records.forEach(x->{
                DataAcquisitionInstrumentVo instrumentVo = new DataAcquisitionInstrumentVo();
                List<AnalyticalInstruments> analyticalInstruments = analyticalInstrumentsService.getByDataAcquisitionInstrumentid(x.getId());
                BeanUtils.copyProperties(x, instrumentVo);
                if (instrumentVo.getCompanyCode()!=null  && componyMap.get(instrumentVo.getCompanyCode())!=null){
                    Map componyMapobj = (Map)componyMap.get(instrumentVo.getCompanyCode());
                    instrumentVo.setCompanyName(componyMapobj.get("companylongname").toString());
                }
                if (instrumentVo.getDepartmentCode()!=null && departmentMap.get(instrumentVo.getDepartmentCode())!=null){
                    Map componyMapobj = (Map)departmentMap.get(instrumentVo.getDepartmentCode());
                    instrumentVo.setDepartmentName(componyMapobj.get("departmentname").toString());
                };
                if (!analyticalInstruments.isEmpty()){
                    List<AnalyticalInstrumentsVo> analyticalInstrumentList = new ArrayList<>();
                    analyticalInstruments.forEach(y->{
                        AnalyticalInstrumentsVo analyticalInstrumentsVo = new AnalyticalInstrumentsVo();
                        BeanUtils.copyProperties(y, analyticalInstrumentsVo);
                        analyticalInstrumentList.add(analyticalInstrumentsVo);
                        List<DetectMonitoringFactor> analyticalInstrumentId = detectMonitoringFactorService.listByParentId(y.getId());
                        if (!analyticalInstrumentId.isEmpty()){
                            analyticalInstrumentId.forEach(z->{
                                analyticalInstrumentsVo.getDetectMonitoringFactors().add(z);
                            });
                        }
                    });
                    instrumentVo.setAnalyticalInstruments(analyticalInstrumentList);
                }
                revoList.add(instrumentVo);
            });
            volits.setRecords(revoList);
        }
        return success(volits);
    }

    /**
     * 根据菜单编号获取详细信息
     */
    @GetMapping(value = "delete/{id}")
    public AjaxResult delete(@PathVariable String id)
    {
        DataAcquisitionInstrument dataAcquisitionInstrument = dataAcquisitionInstrumentService.getById(id);
        DataAcquisitionInstrumentVo instrumentVo = new DataAcquisitionInstrumentVo();
        if (dataAcquisitionInstrument != null){
            BeanUtils.copyProperties(dataAcquisitionInstrument, instrumentVo);
            List<AnalyticalInstruments> analyticalInstruments = analyticalInstrumentsService.getByDataAcquisitionInstrumentid(dataAcquisitionInstrument.getId());
            if (!analyticalInstruments.isEmpty()){
                List<AnalyticalInstrumentsVo> analyticalInstrumentList = new ArrayList<>();
                analyticalInstruments.forEach(y->{
                    if (y.getId() != null)
                    detectMonitoringFactorService.remove(new QueryWrapper<DetectMonitoringFactor>().eq("analytical_instrument_id", y.getId()));
                });
                if (analyticalInstruments.isEmpty())
                analyticalInstrumentsService.remove(new QueryWrapper<AnalyticalInstruments>().eq("data_acquisition_instrument_id", dataAcquisitionInstrument.getId()));
                instrumentVo.setAnalyticalInstruments(analyticalInstrumentList);
            }
        }
        dataAcquisitionInstrumentService.remove(new QueryWrapper<DataAcquisitionInstrument>().eq("id", id));
        return success();
    }

    /**
     * 根据菜单编号获取详细信息
     */
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable String id)
    {
        DataAcquisitionInstrument dataAcquisitionInstrument = dataAcquisitionInstrumentService.getById(id);
        DataAcquisitionInstrumentVo instrumentVo = new DataAcquisitionInstrumentVo();
        if (dataAcquisitionInstrument != null){
            BeanUtils.copyProperties(dataAcquisitionInstrument, instrumentVo);
            List<AnalyticalInstruments> analyticalInstruments = analyticalInstrumentsService.getByDataAcquisitionInstrumentid(dataAcquisitionInstrument.getId());
            if (!analyticalInstruments.isEmpty()){
                List<AnalyticalInstrumentsVo> analyticalInstrumentList = new ArrayList<>();
                analyticalInstruments.forEach(x->{
                        AnalyticalInstrumentsVo analyticalInstrumentsVo = new AnalyticalInstrumentsVo();
                        BeanUtils.copyProperties(x, analyticalInstrumentsVo);

                        List<DetectMonitoringFactor> analyticalInstrumentId = detectMonitoringFactorService.listByParentId(x.getId());
                        if (!analyticalInstrumentId.isEmpty()){
                            analyticalInstrumentId.forEach(z->{
                                analyticalInstrumentsVo.getDetectMonitoringFactors().add(z);
                            });
                        }
                        analyticalInstrumentList.add(analyticalInstrumentsVo);
                    });
                instrumentVo.setAnalyticalInstruments(analyticalInstrumentList);
            }
        }
        return success(instrumentVo);
    }


    /**
     * 新增菜单
     */
    @PostMapping("/save")
    @Transactional
    public AjaxResult save(@Validated @RequestBody DataAcquisitionInstrumentVo vo)
    {
        DataAcquisitionInstrument dataAcquisitionInstrument = new DataAcquisitionInstrument();
        BeanUtils.copyProperties(vo, dataAcquisitionInstrument);
        boolean save = dataAcquisitionInstrumentService.saveOrUpdate(dataAcquisitionInstrument);
        if (save && vo.getAnalyticalInstruments()!=null){
            List<AnalyticalInstrumentsVo> analyticalInstrumentList = vo.getAnalyticalInstruments();
            // 删除子项 全删全增操作
            {
                if (dataAcquisitionInstrument.getId() != null){
                    List<AnalyticalInstruments> dataAcquisitionInstrumentId = analyticalInstrumentsService.getByDataAcquisitionInstrumentid(dataAcquisitionInstrument.getId());
                    if (!dataAcquisitionInstrumentId.isEmpty()){
                        analyticalInstrumentList.forEach(y->{
                            detectMonitoringFactorService.remove(new QueryWrapper<DetectMonitoringFactor>().eq("analytical_instrument_id", y.getId()));
                        });
                        analyticalInstrumentsService.remove(new QueryWrapper<AnalyticalInstruments>().eq("data_acquisition_instrument_id", dataAcquisitionInstrument.getId()));
                    }
                }
            }
            // 新增子表
            analyticalInstrumentList.forEach(x->{
                AnalyticalInstruments analyticalInstruments = new AnalyticalInstruments();
                BeanUtils.copyProperties(x, analyticalInstruments);
                analyticalInstruments.setDataAcquisitionInstrumentId(dataAcquisitionInstrument.getId());
                boolean save1 = analyticalInstrumentsService.save(analyticalInstruments);
                if (save1){
                    List<DetectMonitoringFactor> detectMonitoringFactors = x.getDetectMonitoringFactors();
                    detectMonitoringFactors.forEach(y->{
                        y.setAnalyticalInstrumentId(analyticalInstruments.getId());
                        boolean save2 = detectMonitoringFactorService.save(y);
                        if (!save2){
                            throw new RuntimeException("添加失败");
                        }
                    });
               }
            });
        }
        return success();
    }

    @GetMapping(value = "getCompony")
    public AjaxResult getCompony()
    {
        List<Map<String,Object>> list = dataAcquisitionInstrumentService.getCompony();
        return success(list);
    }


    /**
     * 部门
     * @return
     */
    @GetMapping(value = "getDepartment/{componyid}")
    public AjaxResult getDepartment(@PathVariable String componyid)
    {
        List<Map<String,Object>> list = dataAcquisitionInstrumentService.getDepartment(componyid);
        return success(list);
    }


}
